=== modified file 'DBUS-API'
--- DBUS-API	2015-05-23 20:18:34 +0000
+++ DBUS-API	2014-08-10 14:13:02 +0000
@@ -134,10 +134,10 @@
    k) A raw byte array, not hexadecimal digits.
 
 ** Signals
-*** CheckerCompleted(n: Exitcode, x: Signal, s: Command)
+*** CheckerCompleted(n: Exitcode, x: Waitstatus, s: Command)
     A checker (Command) has completed.  Exitcode is either the exit
-    code or -1 for abnormal exit, in which case, the signal number
-    is available.
+    code or -1 for abnormal exit.  In any case, the full Waitstatus
+    (as from wait(2)) is also available.
     
 *** CheckerStarted(s: Command)
     A checker command (Command) has just been started.
@@ -155,8 +155,8 @@
 
 * Copyright
 
-    Copyright © 2010-2015 Teddy Hogeborn
-    Copyright © 2010-2015 Björn Påhlsson
+    Copyright © 2010-2012 Teddy Hogeborn
+    Copyright © 2010-2012 Björn Påhlsson
   
 ** License:
    

=== modified file 'mandos'
--- mandos	2015-05-23 20:18:34 +0000
+++ mandos	2015-05-31 16:13:39 +0000
@@ -78,6 +78,7 @@
 import tempfile
 import itertools
 import collections
+import codecs
 
 import dbus
 import dbus.service
@@ -423,15 +424,6 @@
             .format(self.name)))
         return ret
 
-def subprocess_call_pipe(connection, # : multiprocessing.Connection
-                         *args, **kwargs):
-    """This function is meant to be called by multiprocessing.Process
-    
-    This function runs a synchronous subprocess.call(), and writes the
-    resulting return code on the provided multiprocessing.Connection.
-    """
-    connection.send(subprocess.call(*args, **kwargs))
-    connection.close()
 
 class Client(object):
     """A representation of a client host served by this server.
@@ -464,8 +456,6 @@
     last_checker_status: integer between 0 and 255 reflecting exit
                          status of last checker. -1 reflects crashed
                          checker, -2 means no checker completed yet.
-    last_checker_signal: The signal which killed the last checker, if
-                         last_checker_status is -1
     last_enabled: datetime.datetime(); (UTC) or None
     name:       string; from the config file, used in log messages and
                         D-Bus identifiers
@@ -645,18 +635,12 @@
         # Also start a new checker *right now*.
         self.start_checker()
     
-    def checker_callback(self, source, condition,
-                         (connection, command)):
+    def checker_callback(self, pid, condition, command):
         """The checker has completed, so take appropriate actions."""
         self.checker_callback_tag = None
         self.checker = None
-        # Read return code from connection (see subprocess_call_pipe)
-        returncode = connection.recv()
-        connection.close()
-        
-        if returncode >= 0:
-            self.last_checker_status = returncode
-            self.last_checker_signal = None
+        if os.WIFEXITED(condition):
+            self.last_checker_status = os.WEXITSTATUS(condition)
             if self.last_checker_status == 0:
                 logger.info("Checker for %(name)s succeeded",
                             vars(self))
@@ -665,16 +649,13 @@
                 logger.info("Checker for %(name)s failed", vars(self))
         else:
             self.last_checker_status = -1
-            self.last_checker_signal = -returncode
             logger.warning("Checker for %(name)s crashed?",
                            vars(self))
-        return False
     
     def checked_ok(self):
         """Assert that the client has been seen, alive and well."""
         self.last_checked_ok = datetime.datetime.utcnow()
         self.last_checker_status = 0
-        self.last_checker_signal = None
         self.bump_timeout()
     
     def bump_timeout(self, timeout=None):
@@ -706,10 +687,20 @@
         # than 'timeout' for the client to be disabled, which is as it
         # should be.
         
-        if self.checker is not None and not self.checker.is_alive():
-            logger.warning("Checker was not alive; joining")
-            self.checker.join()
-            self.checker = None
+        # If a checker exists, make sure it is not a zombie
+        try:
+            pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
+        except AttributeError:
+            pass
+        except OSError as error:
+            if error.errno != errno.ECHILD:
+                raise
+        else:
+            if pid:
+                logger.warning("Checker was a zombie")
+                gobject.source_remove(self.checker_callback_tag)
+                self.checker_callback(pid, status,
+                                      self.current_checker_command)
         # Start a new checker if needed
         if self.checker is None:
             # Escape attributes for the shell
@@ -724,30 +715,46 @@
                              exc_info=error)
                 return True     # Try again later
             self.current_checker_command = command
-            logger.info("Starting checker %r for %s", command,
-                        self.name)
-            # We don't need to redirect stdout and stderr, since
-            # in normal mode, that is already done by daemon(),
-            # and in debug mode we don't want to.  (Stdin is
-            # always replaced by /dev/null.)
-            # The exception is when not debugging but nevertheless
-            # running in the foreground; use the previously
-            # created wnull.
-            popen_args = { "close_fds": True,
-                           "shell": True,
-                           "cwd": "/" }
-            if (not self.server_settings["debug"]
-                and self.server_settings["foreground"]):
-                popen_args.update({"stdout": wnull,
-                                   "stderr": wnull })
-            pipe = multiprocessing.Pipe(duplex=False)
-            self.checker = multiprocessing.Process(
-                target=subprocess_call_pipe, args=(pipe[1], command),
-                kwargs=popen_args)
-            self.checker.start()
-            self.checker_callback_tag = gobject.io_add_watch(
-                pipe[0].fileno(), gobject.IO_IN,
-                self.checker_callback, (pipe[0], command))
+            try:
+                logger.info("Starting checker %r for %s", command,
+                            self.name)
+                # We don't need to redirect stdout and stderr, since
+                # in normal mode, that is already done by daemon(),
+                # and in debug mode we don't want to.  (Stdin is
+                # always replaced by /dev/null.)
+                # The exception is when not debugging but nevertheless
+                # running in the foreground; use the previously
+                # created wnull.
+                popen_args = {}
+                if (not self.server_settings["debug"]
+                    and self.server_settings["foreground"]):
+                    popen_args.update({"stdout": wnull,
+                                       "stderr": wnull })
+                self.checker = subprocess.Popen(command,
+                                                close_fds=True,
+                                                shell=True,
+                                                cwd="/",
+                                                **popen_args)
+            except OSError as error:
+                logger.error("Failed to start subprocess",
+                             exc_info=error)
+                return True
+            self.checker_callback_tag = gobject.child_watch_add(
+                self.checker.pid, self.checker_callback, data=command)
+            # The checker may have completed before the gobject
+            # watch was added.  Check for this.
+            try:
+                pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
+            except OSError as error:
+                if error.errno == errno.ECHILD:
+                    # This should never happen
+                    logger.error("Child process vanished",
+                                 exc_info=error)
+                    return True
+                raise
+            if pid:
+                gobject.source_remove(self.checker_callback_tag)
+                self.checker_callback(pid, status, command)
         # Re-run this periodically if run by gobject.timeout_add
         return True
     
@@ -759,7 +766,14 @@
         if getattr(self, "checker", None) is None:
             return
         logger.debug("Stopping checker for %(name)s", vars(self))
-        self.checker.terminate()
+        try:
+            self.checker.terminate()
+            #time.sleep(0.5)
+            #if self.checker.poll() is None:
+            #    self.checker.kill()
+        except OSError as error:
+            if error.errno != errno.ESRCH: # No such process
+                raise
         self.checker = None
 
 
@@ -1342,24 +1356,24 @@
             DBusObjectWithProperties.__del__(self, *args, **kwargs)
         Client.__del__(self, *args, **kwargs)
     
-    def checker_callback(self, source, condition,
-                         (connection, command), *args, **kwargs):
-        ret = Client.checker_callback(self, source, condition,
-                                      (connection, command), *args,
-                                      **kwargs)
-        exitstatus = self.last_checker_status
-        if exitstatus >= 0:
+    def checker_callback(self, pid, condition, command,
+                         *args, **kwargs):
+        self.checker_callback_tag = None
+        self.checker = None
+        if os.WIFEXITED(condition):
+            exitstatus = os.WEXITSTATUS(condition)
             # Emit D-Bus signal
             self.CheckerCompleted(dbus.Int16(exitstatus),
-                                  dbus.Int64(0),
+                                  dbus.Int64(condition),
                                   dbus.String(command))
         else:
             # Emit D-Bus signal
             self.CheckerCompleted(dbus.Int16(-1),
-                                  dbus.Int64(
-                                      self.last_checker_signal),
+                                  dbus.Int64(condition),
                                   dbus.String(command))
-        return ret
+        
+        return Client.checker_callback(self, pid, condition, command,
+                                       *args, **kwargs)
     
     def start_checker(self, *args, **kwargs):
         old_checker_pid = getattr(self.checker, "pid", None)
@@ -1658,7 +1672,7 @@
         self._pipe = child_pipe
         self._pipe.send(('init', fpr, address))
         if not self._pipe.recv():
-            raise KeyError()
+            raise KeyError(fpr)
     
     def __getattribute__(self, name):
         if name == '_pipe':
@@ -2168,15 +2182,6 @@
     # avoid excessive use of external libraries.
     
     # New type for defining tokens, syntax, and semantics all-in-one
-    Token = collections.namedtuple("Token",
-                                   ("regexp", # To match token; if
-                                              # "value" is not None,
-                                              # must have a "group"
-                                              # containing digits
-                                    "value",  # datetime.timedelta or
-                                              # None
-                                    "followers")) # Tokens valid after
-                                                  # this token
     Token = collections.namedtuple("Token", (
         "regexp",  # To match token; if "value" is not None, must have
                    # a "group" containing digits
@@ -2217,7 +2222,7 @@
     # Define starting values
     value = datetime.timedelta() # Value so far
     found_token = None
-    followers = frozenset((token_duration,)) # Following valid tokens
+    followers = frozenset((token_duration, )) # Following valid tokens
     s = duration                # String left to parse
     # Loop until end token is found
     while found_token is not token_end:
@@ -2240,7 +2245,8 @@
                 break
         else:
             # No currently valid tokens were found
-            raise ValueError("Invalid RFC 3339 duration")
+            raise ValueError("Invalid RFC 3339 duration: {!r}"
+                             .format(duration))
     # End token found
     return value
 
@@ -2497,7 +2503,7 @@
             pidfilename = "/var/run/mandos.pid"
         pidfile = None
         try:
-            pidfile = open(pidfilename, "w")
+            pidfile = codecs.open(pidfilename, "w", encoding="utf-8")
         except IOError as e:
             logger.error("Could not open file %r", pidfilename,
                          exc_info=e)
@@ -2562,7 +2568,7 @@
             old_bus_name = dbus.service.BusName(
                 "se.bsnet.fukt.Mandos", bus,
                 do_not_queue=True)
-        except dbus.exceptions.NameExistsException as e:
+        except dbus.exceptions.DBusException as e:
             logger.error("Disabling D-Bus:", exc_info=e)
             use_dbus = False
             server_settings["use_dbus"] = False
@@ -2641,7 +2647,7 @@
                     pass
             
             # Clients who has passed its expire date can still be
-            # enabled if its last checker was successful.  A Client
+            # enabled if its last checker was successful.  Clients
             # whose checker succeeded before we stored its state is
             # assumed to have successfully run all checkers during
             # downtime.
@@ -2699,10 +2705,10 @@
     
     if not foreground:
         if pidfile is not None:
+            pid = os.getpid()
             try:
                 with pidfile:
-                    pid = os.getpid()
-                    pidfile.write("{}\n".format(pid).encode("utf-8"))
+                    print(pid, file=pidfile)
             except IOError:
                 logger.error("Could not write to file %r with PID %d",
                              pidfilename, pid)

=== modified file 'mandos-ctl'
--- mandos-ctl	2015-04-02 21:56:15 +0000
+++ mandos-ctl	2015-05-31 16:13:39 +0000
@@ -111,15 +111,11 @@
     # avoid excessive use of external libraries.
     
     # New type for defining tokens, syntax, and semantics all-in-one
-    Token = collections.namedtuple("Token",
-                                   ("regexp", # To match token; if
-                                              # "value" is not None,
-                                              # must have a "group"
-                                              # containing digits
-                                    "value",  # datetime.timedelta or
-                                              # None
-                                    "followers")) # Tokens valid after
-                                                  # this token
+    Token = collections.namedtuple("Token", (
+        "regexp",  # To match token; if "value" is not None, must have
+                   # a "group" containing digits
+        "value",   # datetime.timedelta or None
+        "followers"))           # Tokens valid after this token
     # RFC 3339 "duration" tokens, syntax, and semantics; taken from
     # the "duration" ABNF definition in RFC 3339, Appendix A.
     token_end = Token(re.compile(r"$"), None, frozenset())
@@ -178,7 +174,8 @@
                 break
         else:
             # No currently valid tokens were found
-            raise ValueError("Invalid RFC 3339 duration")
+            raise ValueError("Invalid RFC 3339 duration: {!r}"
+                             .format(duration))
     # End token found
     return value
 
@@ -186,17 +183,17 @@
 def string_to_delta(interval):
     """Parse a string and return a datetime.timedelta
     
-    >>> string_to_delta("7d")
+    >>> string_to_delta('7d')
     datetime.timedelta(7)
-    >>> string_to_delta("60s")
+    >>> string_to_delta('60s')
     datetime.timedelta(0, 60)
-    >>> string_to_delta("60m")
+    >>> string_to_delta('60m')
     datetime.timedelta(0, 3600)
-    >>> string_to_delta("24h")
+    >>> string_to_delta('24h')
     datetime.timedelta(1)
-    >>> string_to_delta("1w")
+    >>> string_to_delta('1w')
     datetime.timedelta(7)
-    >>> string_to_delta("5m 30s")
+    >>> string_to_delta('5m 30s')
     datetime.timedelta(0, 330)
     """
     

=== modified file 'mandos-monitor'
--- mandos-monitor	2015-05-23 20:18:34 +0000
+++ mandos-monitor	2014-10-05 20:08:58 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2009-2015 Teddy Hogeborn
-# Copyright © 2009-2015 Björn Påhlsson
+# Copyright © 2009-2014 Teddy Hogeborn
+# Copyright © 2009-2014 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -173,7 +173,7 @@
             gobject.source_remove(self._update_timer_callback_tag)
             self._update_timer_callback_tag = None
     
-    def checker_completed(self, exitstatus, signal, command):
+    def checker_completed(self, exitstatus, condition, command):
         if exitstatus == 0:
             self.logger('Checker for client {} (command "{}")'
                         ' succeeded'.format(self.properties["Name"],
@@ -181,16 +181,20 @@
             self.update()
             return
         # Checker failed
-        if exitstatus >= 0:
+        if os.WIFEXITED(condition):
             self.logger('Checker for client {} (command "{}") failed'
                         ' with exit code {}'
                         .format(self.properties["Name"], command,
-                                exitstatus))
-        elif signal != 0:
+                                os.WEXITSTATUS(condition)))
+        elif os.WIFSIGNALED(condition):
             self.logger('Checker for client {} (command "{}") was'
                         ' killed by signal {}'
                         .format(self.properties["Name"], command,
-                                signal))
+                                os.WTERMSIG(condition)))
+        elif os.WCOREDUMP(condition):
+            self.logger('Checker for client {} (command "{}") dumped'
+                        ' core'.format(self.properties["Name"],
+                                       command))
         else:
             self.logger('Checker for client {} completed'
                         ' mysteriously'

